@@ -71,8 +71,6 @@ static GPtrArray      *search_by_uids        (CamelFolder *folder,
 					      const char *expression,
 					      GPtrArray *uids,
 					      CamelException *ex);
-static void            search_free           (CamelFolder *folder,
-					      GPtrArray *uids);
 static void            transfer_messages_to  (CamelFolder *source,
 					      GPtrArray *uids,
 					      CamelFolder *dest,
@@ -102,7 +100,8 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->get_message = get_message;
 	camel_folder_class->search_by_expression = search_by_expression;
 	camel_folder_class->search_by_uids = search_by_uids;
-	camel_folder_class->search_free = search_free;
+	/* use the default function for the search_free */
+	/* camel_folder_class->search_free = search_free; */
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 	camel_folder_class->refresh_info = refresh_info;
 	camel_folder_class->sync = exchange_sync;
@@ -175,14 +174,28 @@ refresh_info (CamelFolder *folder, CamelException *ex)
 {
 	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
 	CamelExchangeStore *store = CAMEL_EXCHANGE_STORE (folder->parent_store);
+	guint32 unread_count, visible_count;
 
 	if (camel_exchange_store_connected (store, ex)) {
 		camel_offline_journal_replay (exch->journal, NULL);
 
 		camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_REFRESH_FOLDER,
 				 CAMEL_STUB_ARG_FOLDER, folder->full_name,
-				 CAMEL_STUB_ARG_END);
+			     	 CAMEL_STUB_ARG_END);
 	}
+	
+	/* sync up the counts now */
+	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_SYNC_COUNT,
+				 CAMEL_STUB_ARG_FOLDER, folder->full_name,
+				 CAMEL_STUB_ARG_RETURN,
+			      	 CAMEL_STUB_ARG_UINT32, &unread_count,
+				 CAMEL_STUB_ARG_UINT32, &visible_count,
+			      	 CAMEL_STUB_ARG_END)){
+		g_print("\n Error syncing up the counts");
+	}
+
+	folder->summary->unread_count = unread_count;
+	folder->summary->visible_count = visible_count;
 }
 
 static void
@@ -465,24 +478,15 @@ search_by_expression (CamelFolder *folder, const char *expression,
 		      CamelException *ex)
 {
 	CamelFolderSearch *search;
-	GPtrArray *matches, *response;
-	int i;
+	GPtrArray *matches;
 
 	search = camel_exchange_search_new ();
 	camel_folder_search_set_folder (search, folder);
 	matches = camel_folder_search_search (search, expression, NULL, ex);
 
-	if (matches) {
-		response = g_ptr_array_new ();
-		for (i = 0; i < matches->len; i++)
-			g_ptr_array_add (response, g_strdup (matches->pdata[i]));
-		camel_folder_search_free_result (search, matches);
-	} else
-		response = NULL;
-
 	camel_object_unref (CAMEL_OBJECT (search));
 
-	return response;
+	return matches;
 }
 
 static GPtrArray *
@@ -502,16 +506,6 @@ search_by_uids (CamelFolder *folder, const char *expression,
 	return matches;
 }
 
-static void
-search_free (CamelFolder *folder, GPtrArray *uids)
-{
-	int i;
-
-	for (i = 0; i < uids->len; i++)
-		g_free (uids->pdata[i]);
-	g_ptr_array_free (uids, TRUE);
-}
-
 static void
 transfer_messages_the_hard_way (CamelFolder *source, GPtrArray *uids,
 				CamelFolder *dest,
@@ -728,6 +722,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	CamelFolderChangeInfo *changes;
 	CamelStream *stream;
 	CamelMimeMessage *msg;
+	guint32 unread_count, visible_count;
 
 	info = camel_folder_summary_uid (folder->summary, uid);
 	if (info) {
@@ -767,6 +762,11 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 
 	camel_folder_summary_add (folder->summary, info);
 
+	if (!(flags & CAMEL_MESSAGE_SEEN)) {
+		folder->summary->unread_count++;
+		folder->summary->visible_count++;
+	}
+
 	changes = camel_folder_change_info_new ();
 	camel_folder_change_info_add_uid (changes, uid);
 	camel_folder_change_info_recent_uid (changes, uid);
@@ -1054,7 +1054,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 		if (summary->len - camel_folder_summary_cache_size (folder->summary) > 50)
 			camel_folder_summary_reload_from_db (folder->summary, ex);
-		
+
 		for (i = 0; i < summary->len; i++) {
 			uids->pdata[i] = g_strdup(summary->pdata[i]);
 			info = camel_folder_summary_uid (folder->summary, uids->pdata[i]);
@@ -1103,7 +1103,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		camel_operation_end (NULL);
 		if (!ok)
 			return FALSE;
-		
+
 		camel_folder_summary_save_to_db (folder->summary, ex);
 	}
 
